/**
 ** Copyright (c) 2003-2010 LAAS/CNRS 
 **
 ** Matthieu Herrb - Mon Apr 14 2003
 **
 ** $Source$
 ** $Revision$
 ** $Date$
 ** 
 **/

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include "serial.h"

static void mycfmakeraw(struct termios *);
#ifdef __sun 
static int cfsetspeed(struct termios *, speed_t);
#endif

int 
openSerial(const char *device, int speed)
{
	int fd, flags;
	struct termios t;

	fd = open(device, O_RDWR|O_NONBLOCK);
	if (fd < 0) {
		fprintf(stderr, "%s:%d: %s: %s\n", __FILE__, __LINE__,
		    device, strerror(errno));
		return -1;
	}
	printf ("openSerial: open %s ok fd = %d\n", device, fd);
	tcgetattr(fd, &t);
	mycfmakeraw(&t);

	switch (speed) {
	case 1200:
		cfsetspeed(&t, B1200);
		break;
	case 2400:
		cfsetspeed(&t, B2400);
		break;
	case 4800:
		cfsetspeed(&t, B4800);
		break;
	case 9600:
		cfsetspeed(&t, B9600);
		break;
	case 19200:
		cfsetspeed(&t, B19200);
		break;
	case 38400:
		cfsetspeed(&t, B38400);
		break;
	case 57600:
		cfsetspeed(&t, B57600);
		break;
	case 115200:
		cfsetspeed(&t, B115200);
		break;
	default:
		fprintf(stderr, "%s:%d: unknown speed %d\n", __FILE__,
		    __LINE__, speed);
		close(fd);
		return -1;
	}
	if (tcsetattr(fd, TCSAFLUSH, &t) < 0) {
		fprintf(stderr, "%s:%d: tcsetattr: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		close (fd);
		return -1;
	}
	flags = fcntl(fd, F_GETFL, 0);
	if (flags < 0) {
		fprintf(stderr, "%s:%d: F_GETFL: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		close (fd);
		return -1;
	}
	flags &= ~O_NONBLOCK;
	if (fcntl(fd, F_SETFL, flags) < 0) {
		fprintf(stderr, "%s:%d: F_SETFL: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		close (fd);
		return -1;
	}
	
	return fd;
}

int
serialGetChar(int fd)
{
	char buf;
	int n;
	
	errno = 0;
	do {
		n = read(fd, &buf, 1);
		if (n == 0) fprintf(stderr, "nothing...\n");
	} while ((n < 0 && errno == EINTR) || n == 0);

	if (n < 0) {
		fprintf(stderr, "%s:%d: read: %s\n",
		    __FILE__, __LINE__, strerror(errno));
		return -1;
	}
	return (int)buf;
}

int
readSerial(int fd, char *buf, size_t len)
{
#if 1
	int i = 0, n;
	char c;
	
	do {
		errno = 0;
		do {
			n = read(fd, &c, 1);
		} while (n < 0 && errno == EINTR && i < len);
		
		buf[i++] = c;
		if (c == '\n' || i == len) 
			break;
	} while (1);
	return i;
#else
	return read(fd, buf, len);
#endif
}

int 
writeSerial(int fd, char *buf, size_t len)
{
	/* for now */
	return write(fd, buf, len);
}

/**
 ** These functions are missing in Solaris 
 **/
static void
mycfmakeraw(struct termios *t)
{
        t->c_iflag &= ~(IMAXBEL|IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
        t->c_oflag &= ~OPOST;
        t->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
        t->c_cflag &= ~(CSIZE|PARENB|CRTSCTS);
        t->c_cflag |= CS8|CLOCAL;
	t->c_cc[VMIN] = 1;
	t->c_cc[VTIME] = 0;

        /* XXX set MIN/TIME */
}

#ifdef __sun 
static int
cfsetspeed(struct termios *t, speed_t speed)
{
	cfsetispeed(t, speed);
	cfsetospeed(t, speed);
        return (0);
}
#endif
