/**
 ** Copyright (c) 2000-2010 LAAS/CNRS 
 **
 ** Anthony Mallet - Tue Aug  1 2000
 **
 ** $Log$
 ** Revision 1.3  2008/07/10 11:35:28  adegroot
 ** export gyroResetAngle to external world via gyrolib
 ** Implement it for gyro KVH5000
 **
 ** Revision 1.2  2008/07/09 12:15:11  adegroot
 ** Export gyroUpdateWOffset to external application (in particular genom).
 ** Provide a dummy implementation of gyroUpdateWOffset for gyroKVH5000
 **
 ** Revision 1.1  2006/12/02 17:43:27  sara
 ** Library to control KHV1000 FOG gyro (openrobots edition)
 **
 ** Revision 1.2  2005/03/24 18:35:31  sara
 ** Reorganisation of gyro libraries : gyroKVH1000/ and gyroKVH5000/
 ** have been regrouped in a common gyro/ directory.
 ** In such a way the common API gyroLib.h is in the common directory.
 **
 ** Revision 1.1  2005/03/24 17:03:46  sara
 ** Completely new organisation of libraries for the KVH1000 FOG control.
 ** Motivated by pbs with the rocket port. See README files.
 **
 ** Revision 1.4  2004/10/12 15:35:29  sara
 ** Unix also
 **
 ** Revision 1.3  2001/02/21 15:49:49  mallet
 ** Merge simon+sara
 **
 ** Revision 1.2  2001/02/12 09:01:06  sara
 ** o Calcul de l'angle
 ** o Ajout de nouvelles fonctions :
 **  * gyroStop: stop the driver
 **  * gyroResetAngle(double angle): rinitialise l'angle  la valeur voulue
 **  * gyroUpdateWOffset: met  jour l'offset de la vitesse angulaire
 **
 ** Revision 1.1  2000/09/14 18:21:36  mallet
 ** gyro
 **
 **/


#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#ifdef HAS_POSIX_CLOCK
#include <time.h>
#else
#include <sys/time.h>
#endif

#include <portLib.h>
#include <errnoLib.h>
#include <taskLib.h>
#include <semLib.h>
#include <commonStructLib.h>
#include <h2mathLib.h>

#include "../gyroHardLib/serial.h"
#include "gyroKVH1000TaskLib.h"
#include "../gyroHardLib/gyroKVH1000HardLib.h"

#define DEVICE_NAME_LEN	16

/* shared structure */
static GyroData *gyroData = NULL;

static  u_long msec0;

/* local functions */
static void	gyroReadTask(GyroData *gyro);

/*
 * gyroInitAngle -------------------------------------------------------------
 *
 * Initialize gyro
 *
 * WARNING : returns TRUE or FALSE !!
 *
 * (to have same API as for KVH5000)
 */


BOOL
gyroInitAngle(char *device)
{
  if (gyroInit(device) == ERROR) return FALSE;
  if (gyroUpdateWOffset() == FALSE) return FALSE;
  if (gyroResetAngle(0.0) == FALSE) return FALSE;
  fputs("sensorsTask: started\n", stderr);
  return TRUE;
}

/*
 * gyroInit -------------------------------------------------------------
 *
 * Initialize gyro : spawns a thread that reads and decodes permanently
 *                   the gyro data.
 *
 */

STATUS
gyroInit(char *device)
{
  int gyroId;
#ifdef HAS_POSIX_CLOCK
  struct timespec tp; /*  mesure du temps */
#else
  struct timeval tv;
#endif /* HAS_POSIX_CLOCK */


  if (gyroData != NULL) {
    errnoSet(S_gyroLib_ALREADY_INITIALIZED);
    printf ("gyroInit: Already Initialized\n");
    return ERROR;
  }
  
   /* create and initialise shared data */
  if (commonStructCreate(sizeof(GyroData), (void **)&gyroData) != OK) {
    printf ("gyroInit: cannot create commonStruct\n");
    return ERROR;   
  }
  
  commonStructTake(gyroData);
  
  /* record device */
  strncpy(gyroData->device, device, DEVICE_NAME_LEN-1);
  gyroData->device[DEVICE_NAME_LEN-1] = 0;
  
  /* open device */
  if ((gyroData->fd = gyroOpen(gyroData->device)) == -1) {
    printf ("gyro: gyroOpen failed\n");
    commonStructGive(gyroData);
    return ERROR;
  }
  printf ("gyro: open(%s) ok fd =%d\n", gyroData->device, gyroData->fd);
  
  /* read task */
  gyroData->taskId =
    taskSpawn(GYRO_READ_TASK_NAME, GYRO_READ_TASK_PRIORITY, 0,
	      GRYO_READ_TASK_STACK_SIZE,
	      (FUNCPTR)gyroReadTask, gyroData);
  
  gyroData->stop = 0;
  commonStructGive(gyroData);
  if (gyroData->taskId == ERROR) {
    commonStructDelete(gyroData);
    printf ("gyro: taskSpawn failed\n");
      return(ERROR);
  }
  printf ("gyro: taskSpawn ok\n");
  

  gyroId = 0;
  printf ("gyro: wait gyro id ");
  while (!gyroId) {
    /* Test fin toutes les secondes */
    taskDelay(NTICKS_PER_SEC);
    printf (".");
    fflush(stdout);
    commonStructTake(gyroData);
    gyroId = gyroData->snO;
    commonStructGive(gyroData);
  }
  printf ("done\n");
  
#ifdef HAS_POSIX_CLOCK
  clock_gettime(CLOCK_REALTIME, &tp);
  msec0 = (tp.tv_nsec / 1000000) + (tp.tv_sec * 1000);
#else
  gettimeofday(&tv, NULL);
  msec0 = (tv.tv_usec / 1000) + (tv.tv_sec * 1000);
#endif /* HAS_POSIX_CLOCK */
  
  
  return OK;
}


/*
 * gyroEnd  -------------------------------------------------------------
 *
 * End connection to serial port
 *
 * Returns : WARNING : BOOL :  FALSE if failed or TRUE.
 */

BOOL gyroEnd(void)
{
  if (gyroData->fd != -1) {
    if(close(gyroData->fd) == -1) {
      printf ("gyroEnd: close(fd %d) failed\n", gyroData->fd);
      return FALSE;
    }
    printf ("gyroEnd: close(fd %d) OK\n", gyroData->fd);
  }
  return TRUE;
}

/*
 * gyroRate -------------------------------------------------------------
 *
 * returns the angular speed rate and the internal gyro temperature
 */

STATUS
gyroRate(double *rate, double *temperature)
{
   if (gyroData == NULL) {
      errnoSet(S_gyroLib_NOT_INITIALIZED);
      return ERROR;
   }

   commonStructTake(gyroData);
   *rate = gyroData->w;
   *temperature = gyroData->temp;
   commonStructGive(gyroData);
   return OK;
}

/*
 * gyroStop -------------------------------------------------------------
 *
 * stop the driver
 */

STATUS
gyroStop(void)
{
   if (gyroData == NULL) {
      errnoSet(S_gyroLib_NOT_INITIALIZED);
      return ERROR;
   }

   gyroData->stop = 1;

   return OK;
}

/*
 * gyroUpdateWOffset -----------------------------------------------------
 *
 * Computes the w offset (theoritically depends on temperature)
 * Controls also the period
 *
 *         WARNING: the robot must be stopped
 *
 * It returns when the computation is over
 */

#define GYRO_NB_CALIB_MEASURES 200

BOOL
gyroUpdateWOffset(void)
{
  double wOffset;
  int nbMeasuresOffset;

  /* not init */
  if (gyroData == NULL) {
      errnoSet(S_gyroLib_NOT_INITIALIZED);
      return FALSE;
   }

  /* thus need to wait for waitDuration seconds */
  printf ("  gyro period = %ld ms (NTICKS_PER_SEC = %d)\n",
	  gyroData->period,  NTICKS_PER_SEC);
  printf ("  gyroWOffset: needs %d seconds (%d measures) with robot STOPPED !!\n", 
	  (int)(GYRO_NB_CALIB_MEASURES*gyroData->period/1000.0), 
	  GYRO_NB_CALIB_MEASURES);


  /* init mesures */
  commonStructTake(gyroData);
  gyroData->wOffset = 0.0;
  gyroData->nbMeasuresOffset = GYRO_NB_CALIB_MEASURES;
  commonStructGive(gyroData);

  /* mesures */
  nbMeasuresOffset = GYRO_NB_CALIB_MEASURES;
  while (nbMeasuresOffset) {
    /* we do not take the common-struct in purpose to do not disturb
       the main task. this value is just for display */
    nbMeasuresOffset =  gyroData->nbMeasuresOffset;
    printf ("%d ", nbMeasuresOffset);
    fflush(stdout);
    taskDelay(NTICKS_PER_SEC);
  }

  /* print result */
  commonStructTake(gyroData);
  wOffset = gyroData->wOffset;
  commonStructGive(gyroData);
  printf ("\ngyroUpdateWOffset: offset %.3f deg/s\n", wOffset);

  return TRUE;
}

/*
 * gyroResetAngle -------------------------------------------------------------
 *
 * Reset the angle value
 */

BOOL
gyroResetAngle(double angle)
{
   if (gyroData == NULL) {
      errnoSet(S_gyroLib_NOT_INITIALIZED);
      return FALSE;
   }

   commonStructTake(gyroData);
   gyroData->theta = angleLimit(angle);
   gyroData->resetDate = gyroData->date;
   commonStructGive(gyroData);
   return TRUE;
}


/*
 * gyroPeriod -----------------------------------------------------------
 *
 * returns the period of the gyro
 */

STATUS
gyroPeriod(double *period)
{
   if (gyroData == NULL) {
      errnoSet(S_gyroLib_NOT_INITIALIZED);
      return ERROR;
   }

   commonStructTake(gyroData);
   printf ("gyroPeriod: not implemented yet\n");
   *period = 0; /* XXX */
   commonStructGive(gyroData);
   return OK;
}


/*
 * gyroShow -------------------------------------------------------------
 *
 * print gyro data
 */

void
gyroShow(void)
{
  gyroPrint(0);
}

void
gyroPrint(int level)
{
  GyroData str;
    
   if (gyroData == NULL) {
      errnoSet(S_gyroLib_NOT_INITIALIZED);
      printf ("gyroShow: gyro not initialized\n");
      return;
   }

   commonStructCopy(gyroData, TO, &str);

   if (level == 0) {
     printf("\n");
     printf("Gyro device     : %s\n", str.device);
     printf("Read task       : 0x%x\n", str.taskId);
     
     if (str.snO)
       printf("Serial number   : %d\n", str.sn);
     else
       printf("Serial number   : N/A\n");
     printf("Period          : %ld ms\n", str.period);
   }
   printf("Internal temp   : %.2f C\n", str.temp);

   printf("Speed           : %.3f d/s   (max %g d/s offset %.5f d/s)\n", 
	  RAD_TO_DEG(str.w), 
	  GYRO_MAX_ANGULAR_SPEED_DEG_S, 
	  RAD_TO_DEG(str.wOffset));
   printf("Angle           : %.3f deg\n", RAD_TO_DEG(str.theta));
   printf("Duration        : %ld ms\n", str.date - str.resetDate);

   printf("\n");
}

/*
 * gyroReadAngle ---------------------------------------------------------
 *
 * WARNING : return TRUE or FALSE 
 */

BOOL gyroReadAngle(double *theta)
{
  if (gyroData == NULL) {
      errnoSet(S_gyroLib_NOT_INITIALIZED);
      return FALSE;
   }

   commonStructTake(gyroData);
   *theta = gyroData->theta;
   commonStructGive(gyroData);
   return TRUE;
}

/*
 * gyroReadCorrAngle ---------------------------------------------------------
 *
 * Like gyroReadAngle but correct the resulting 
 *                         angle taking into account the earth rotation from 
 *                         the gyroInit().
 *
 * WARNING 1 : to be use only if the gyro is horizontal !
 *
 * WARNING 2 : return TRUE or FALSE 
 * (to have same API as for KVH5000)
 */

BOOL gyroReadCorrAngle(double *theta)
{
  double corrAngle;

  if (gyroData == NULL) {
      errnoSet(S_gyroLib_NOT_INITIALIZED);
      return FALSE;
   }

  commonStructTake(gyroData);
  /* earth rotation */
  corrAngle = (gyroData->date - gyroData->resetDate) / 1000. * 2 * PI / 86400;
  *theta = gyroData->theta - corrAngle;
  *theta = angleLimit(*theta);
  commonStructGive(gyroData);
  return TRUE;
}



/*
 * gyroReadTask ---------------------------------------------------------
 *
 * This function reads and decode gyro data permanently.
 *
 * Thus it must be started within an independant thread.
 *
 */

static void
gyroReadTask(GyroData *gyro)
{
  u_long date;

  fputs("---- gyro: started\n", stderr);

   while(!gyro->stop) {
     if (gyroRead(gyro->fd, gyro->buf, &date) < 0) break;

      commonStructTake(gyro);
      gyro->date = date;
      if (gyroDecode(gyro) < 0) 
	 fputs("gyro: lost message\n", stderr);
      commonStructGive(gyro);
   }

   /* Stop */
   if (close(gyro->fd) == -1) {
     printf ("gyro: cannot close fd %d errno = %d \n", gyro->fd, errno);
   }

   commonStructDelete(gyro);
   gyroData = NULL;

   fputs("gyro: stopped\n", stderr);
}
