/* ----------------------------------------------------------------------
 * Copyright (c) 2005 LAAS/CNRS
 *
 * Sara Fleury - Mon Mar 21 17:15:29 CET 2005  
 * 
 * gyroTest - allows to test gyroServerLib and guroClient to interact 
 *            with a KVH1000 gyro
 *
 * usage: gyroTest  <serial-port>  <calibrate-flag>
 *    eg: gyroTest  /dev/ttyS1  1
 * 
 * C-c to stop
 * 
 */

#include <portLib.h>

#include <stdio.h>
#include <stdlib.h> /* atoi */
#include "taskLib.h" /* taskDelay */
#include "../../gyroLib.h"
#include "../gyroTaskLib/gyroKVH1000TaskLib.h"
#include <h2initGlob.h>

/* ----------------------------------------------------------------------
 *
 * main : gyroTest
 *
 */

int main(int argc, char *argv[])
{
  double wGyro, temperature;
  int calibrate=0;

  /* get parameters */
  if (argc < 2 || argc >3 || argv[1][0] == '-') {
    printf ("usage: gyroTest  <serial-port>  <calibrate-flag>\n"
	    "   eg: gyroTest  /dev/ttyS1  1\n");
    return -1;
  }
  if (argc==3)
    calibrate = atoi(argv[2]);

  h2initGlob(0);

  /* initialize gyroLib */
  if (gyroInit(argv[1]) != OK) {
    fputs("gyroTest: could not initialize gyro\n", stderr);
    return ERROR;
  }

  /* calibrate offset if required */
  if (calibrate) {
    fputs("gyroTest: calibration ...\n", stderr);
    
    gyroUpdateWOffset();
    
    fputs("gyroTest: ... ok.\n"
	  "gyroTest: resetAngle.\n", stderr);
  }

  /* reset angle to 0 */
  gyroResetAngle(0.0);

  fputs("gyroTest: started !\n", stderr);

  /* reads periodically */
  while (1) {
    
    taskDelay(100);
  
    /* read gyro */
    if (gyroRate(&wGyro, &temperature) != OK) {
     fputs("gyroTest: gyroRead error\n", stderr);
    }

    /* print gyro */
    gyroPrint(1);
  }

  gyroEnd();

   return 0;
}
