/*
 * Copyright (c) 2004 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.
 */

#include <stdio.h>
#include <errno.h>
#include <signal.h>

#include <portLib.h>
#include <h2initGlob.h>
#include <taskLib.h>
#include <h2mathLib.h>

#include "../gyroLib.h"

#define TEST_WITH_GYRO

/* ----------------------------------------------------------------------
 *
 * gyroTest - init gyro KVH50000 and read and print angle
 *
 */

static int stop = 0;
static int stopped = 0;

void stop_handler (int sig)
{
  printf ("free serial...\n");
  stop = 1;
#ifdef TEST_WITH_GYRO
  taskDelay(100);
  gyroEnd();
#endif
  printf ("bye\n");
}

int main()
{
  double theta;
  int count=1;
  char dev[64];

  h2initGlob(100);

  

  printf ("port device name:");
  fflush(stdout);
  scanf("%s", dev);

  printf ("Init with %s\n", dev);

#ifdef TEST_WITH_GYRO
  /* initialize gyroLib */
  if (!gyroInitAngle(dev)) {
    fputs("gyroInitAngle: could not initialize gyro\n", stderr);
    return ERROR;
  }
#endif

  fputs("gyroInitAngle: done\n", stderr);

  printf ("TO STOP:  C-c  \n");

  signal(SIGINT,stop_handler);

  while (!stop) {
    
    taskDelay(100);

#ifdef TEST_WITH_GYRO  
    /* read gyro */
    if (!gyroReadCorrAngle(&theta)) {
     fputs("gyroKVH5000Test: gyroReadCorrAngle failed\n", stderr);
    }
/*     printf ("theta %f\n",theta); */
#endif

    if (count==1) {
      printf ("gyroTest: %6.3f deg = %7.4f rad\n", RAD_TO_DEG(theta), theta);
      count=0;
    }
    count++;
    
  }

  stopped = 1;
  
  return 0;
}
